VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "clsDrawingSurface"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
'********************************************
'* clsDrawingSurface.cls                    *
'* Version 1.1                              *
'* Enables extra drawing surfaces to be     *
'* created and used                         *
'********************************************
Option Explicit

'Surface variables
Public Surface As DirectDrawSurface7
Dim SurfaceDesc As DDSURFACEDESC2
Dim SurfaceRect As DxVBLib.RECT
Dim ColourKey As DDCOLORKEY

'Class variables
Public TransparencyEnabled As Boolean

'*****************************
'Class initialize and terminate
'*****************************
Private Sub Class_Terminate()
    Set Surface = Nothing
End Sub

'******************************
'Class Methods
'******************************
Public Function Initialize(Width As Long, Height As Long, ByRef DirectDrawEngine As clsDirectDraw) As Long
    On Error GoTo init_error
    Err.Clear
    
    SurfaceDesc.lFlags = DDSD_CAPS Or DDSD_HEIGHT Or DDSD_WIDTH
    SurfaceDesc.ddsCaps.lCaps = DDSCAPS_OFFSCREENPLAIN
    SurfaceDesc.lWidth = Width
    SurfaceDesc.lHeight = Height
    
    Set Surface = DirectDrawEngine.DirectDraw.CreateSurface(SurfaceDesc)
    If Surface Is Nothing Then GoTo init_error
    
    Surface.GetSurfaceDesc SurfaceDesc
    Surface.SetColorKey DDCKEY_SRCBLT, ColourKey
    With SurfaceRect
        .Left = 0
        .Top = 0
        .Right = SurfaceDesc.lWidth
        .Bottom = SurfaceDesc.lHeight
    End With
    
    Initialize = 0
    Exit Function
init_error:
    Initialize = Err.Number
End Function

Public Sub ResetRectangle()
    With SurfaceRect
        .Left = 0
        .Top = 0
        .Right = SurfaceDesc.lWidth
        .Bottom = SurfaceDesc.lHeight
    End With
End Sub

Public Function Fill(Colour As Long) As Long
    Fill = Surface.BltColorFill(SurfaceRect, Colour)
End Function

'Surface Blitting Functions
Public Function SurfaceBlt(ByRef SurfaceA As DirectDrawSurface7, SrcRect As DxVBLib.RECT, Optional Transparent As Boolean = False) As Long
    On Error GoTo blt_error
    Err.Clear
    
    If Transparent Then
        SurfaceBlt = Surface.Blt(SurfaceRect, SurfaceA, SrcRect, DDBLT_WAIT Or DDBLT_KEYSRC)
    Else
        SurfaceBlt = Surface.Blt(SurfaceRect, SurfaceA, SrcRect, DDBLT_WAIT)
    End If
    Exit Function
blt_error:
    SurfaceBlt = Err.Number
End Function

Public Function SurfaceBltFast(ByRef SurfaceA As DirectDrawSurface7, SrcRect As DxVBLib.RECT, XPos As Long, YPos As Long, Optional Transparent As Boolean = False) As Long
    On Error GoTo blt_error
    Err.Clear
    If Transparent Then
        SurfaceBltFast = Surface.BltFast(XPos, YPos, SurfaceA, SrcRect, DDBLTFAST_WAIT Or DDBLTFAST_SRCCOLORKEY)
    Else
        SurfaceBltFast = Surface.BltFast(XPos, YPos, SurfaceA, SrcRect, DDBLTFAST_WAIT)
    End If
    Exit Function
blt_error:
    SurfaceBltFast = Err.Number
End Function

'Bitmap blitting functions
Public Function BitmapBlt(ByRef Bitmap As clsBitmap) As Long
    On Error GoTo blt_error
    Err.Clear
    If Bitmap.TransparencyEnabled Then
        BitmapBlt = Surface.Blt(SurfaceRect, Bitmap.Surface, Bitmap.Rectangle, DDBLT_WAIT Or DDBLT_KEYSRC)
    Else
        BitmapBlt = Surface.Blt(SurfaceRect, Bitmap.Surface, Bitmap.Rectangle, DDBLT_WAIT)
    End If
    Exit Function
blt_error:
    BitmapBlt = Err.Number
End Function

Public Function BitmapBltFast(ByRef Bitmap As clsBitmap) As Long
    On Error GoTo blt_error
    Err.Clear
    If Bitmap.TransparencyEnabled Then
        BitmapBltFast = Surface.BltFast(Bitmap.X, Bitmap.Y, Bitmap.Surface, Bitmap.Rectangle, DDBLTFAST_WAIT Or DDBLTFAST_SRCCOLORKEY)
    Else
        BitmapBltFast = Surface.BltFast(Bitmap.X, Bitmap.Y, Bitmap.Surface, Bitmap.Rectangle, DDBLTFAST_WAIT)
    End If
    Exit Function
blt_error:
    BitmapBltFast = Err.Number
End Function

'Sprite blitting routines
Public Function SpriteBlt(ByRef Sprite As clsSprite) As Long
    On Error GoTo blt_error
    Err.Clear
    If Sprite.TransparencyEnabled Then
        SpriteBlt = Surface.Blt(SurfaceRect, Sprite.Surface, Sprite.FrameRect(Sprite.CurrentFrame), DDBLT_WAIT Or DDBLT_KEYSRC)
    Else
        SpriteBlt = Surface.Blt(SurfaceRect, Sprite.Surface, Sprite.FrameRect(Sprite.CurrentFrame), DDBLT_WAIT)
    End If
    Exit Function
blt_error:
    SpriteBlt = Err.Number
End Function

Public Function SpriteBltFast(ByRef Sprite As clsSprite) As Long
    On Error GoTo blt_error
    Err.Clear
    If Sprite.TransparencyEnabled Then
        SpriteBltFast = Surface.BltFast(Sprite.X, Sprite.Y, Sprite.Surface, Sprite.FrameRect(Sprite.CurrentFrame), DDBLTFAST_WAIT Or DDBLTFAST_SRCCOLORKEY)
    Else
        SpriteBltFast = Surface.BltFast(Sprite.X, Sprite.Y, Sprite.Surface, Sprite.FrameRect(Sprite.CurrentFrame), DDBLTFAST_WAIT)
    End If
    Exit Function
blt_error:
    SpriteBltFast = Err.Number
End Function

'Basic Drawing functions
Public Function DrawText(Text As String, X As Long, Y As Long) As Long
    On Error GoTo text_error
    Err.Clear
    Surface.DrawText X, Y, Text, False
    DrawText = 0
    Exit Function
text_error:
    DrawText = Err.Number
End Function

Public Sub SetFont(TextFont As StdFont)
    Dim DispFont As IFont
    Set DispFont = TextFont
    Surface.SetFont DispFont
End Sub

Public Function DrawLine(X1 As Long, Y1 As Long, X2 As Long, Y2 As Long) As Long
    On Error GoTo line_error
    Err.Clear
    Surface.DrawLine X1, Y1, X2, Y2
    DrawLine = 0
    Exit Function
line_error:
    DrawLine = Err.Number
End Function

Public Function DrawBox(X1 As Long, Y1 As Long, X2 As Long, Y2 As Long) As Long
    On Error GoTo box_error
    Err.Clear
    Surface.DrawBox X1, Y1, X2, Y2
    DrawBox = 0
    Exit Function
box_error:
    DrawBox = Err.Number
End Function

Public Function DrawRoundedBox(X1 As Long, Y1 As Long, X2 As Long, Y2 As Long, EllipseWidth As Long, EllipseHeight As Long) As Long
    On Error GoTo roundbox_error
    Err.Clear
    Surface.DrawRoundedBox X1, Y1, X2, Y2, EllipseWidth, EllipseHeight
    DrawRoundedBox = 0
    Exit Function
roundbox_error:
    DrawRoundedBox = Err.Number
End Function

Public Function DrawCircle(X As Long, Y As Long, Radius As Long) As Long
    On Error GoTo circle_error
    Err.Clear
    Surface.DrawCircle X, Y, Radius
    DrawCircle = 0
    Exit Function
circle_error:
    DrawCircle = Err.Number
End Function

Public Function DrawEllipse(X1 As Long, Y1 As Long, X2 As Long, Y2 As Long) As Long
    On Error GoTo ellipse_error
    Err.Clear
    Surface.DrawEllipse X1, Y1, X2, Y2
    DrawEllipse = 0
    Exit Function
ellipse_error:
    DrawEllipse = Err.Number
End Function

'********************************
'Class properties
'********************************
Public Property Get Width()
    Width = SurfaceDesc.lWidth
End Property

Public Property Get Height()
    Height = SurfaceDesc.lHeight
End Property

Public Property Let Rectangle(r As DxVBLib.RECT)
    SurfaceRect = r
End Property

Public Property Get Rectangle() As DxVBLib.RECT
    Rectangle = SurfaceRect
End Property

Public Property Let TransparentColour(Colour As DDCOLORKEY)
    ColourKey = Colour
    Surface.SetColorKey DDCKEY_SRCBLT, ColourKey
End Property

Public Property Get TransparentColour() As DDCOLORKEY
    TransparentColour = ColourKey
End Property

Public Property Let ForeColour(Colour As Long)
    Surface.SetForeColor Colour
End Property

Public Property Get ForeColour() As Long
    ForeColour = Surface.GetForeColor
End Property

Public Property Let FillColour(Colour As Long)
    Surface.SetFillColor Colour
End Property

Public Property Get FillColour() As Long
    FillColour = Surface.GetFillColor
End Property

Public Property Let FillStyle(Style As FillStyleConstants)
    Surface.SetFillStyle Style
End Property

Public Property Get FillStyle() As FillStyleConstants
    FillStyle = Surface.GetFillStyle
End Property

Public Property Let DrawStyle(Style As DrawStyleConstants)
    Surface.setDrawStyle Style
End Property

Public Property Get DrawStyle() As DrawStyleConstants
    DrawStyle = Surface.getDrawStyle
End Property

Public Property Let DrawWidth(Width As Long)
    Surface.setDrawWidth Width
End Property

Public Property Get DrawWidth() As Long
    DrawWidth = Surface.getDrawWidth
End Property

